// Generated by CoffeeScript 1.6.3
var Monitor, argv, codeChange, fs, hrLog, hrLogId, logger, moment, notifier, path, project, relativeName, root, script, scriptName, spawn, srcMonitor, srvArgs, start, writeStderr, writeStdout, _, _ref;

fs = require('fs');

path = require('path');

spawn = require('child_process').spawn;

moment = require('moment');

_ = require('lodash');

Monitor = require('findr').Monitor;

argv = require('optimist').alias('d', 'debug').alias('b', 'break').argv;

project = require('./project');

_ref = require('./loggers'), logger = _ref.logger, notifier = _ref.notifier;

script = null;

hrLogId = null;

/*
add an horizontal line after each log to make it easier to read
*/


hrLog = function() {
  var hr, i, _i;
  hr = '';
  for (i = _i = 1; _i <= 50; i = ++_i) {
    hr += '.';
  }
  console.log(hr);
  console.log(moment().format('h:mm:ss - ddd MMM YY'));
  return console.log(hr);
};

writeStdout = function(data) {
  process.stdout.write(data);
  clearTimeout(hrLogId);
  return hrLogId = setTimeout(hrLog, 3000);
};

writeStderr = function(data) {
  process.stderr.write(data);
  clearTimeout(hrLogId);
  return hrLogId = setTimeout(hrLog, 3000);
};

/*
Server stuffs
*/


srvArgs = [];

if (!process.env.NO_SCRIPT) {
  scriptName = process.env.SQ_SCRIPT || project.config.server.script;
}

if (scriptName) {
  start = function(msg) {
    msg || (msg = "Starting script " + scriptName);
    notifier.info(msg, {
      title: scriptName
    });
    if (path.extname(scriptName) === '.js') {
      if (argv.debug) {
        srvArgs.push('--debug');
      }
      if (argv["break"]) {
        srvArgs.push('--debug-brk');
      }
      srvArgs.push(path.resolve(__dirname, 'scriptWrapper.js'));
      script = spawn('node', srvArgs, {
        cwd: '.',
        env: _.extend(process.env, project.config.server.env, {
          SQ_SCRIPT: scriptName
        })
      });
      script.once('exit', function(err) {
        if (!err) {
          return start("Restarting script " + scriptName);
        }
      });
    } else {
      script = spawn(scriptName);
    }
    script.stdout.on('data', writeStdout);
    return script.stderr.on('data', writeStderr);
  };
} else {
  start = function() {};
}

/*
builder stuffs
*/


root = path.resolve('.');

relativeName = function(file) {
  return file != null ? file.substring(root.length) : void 0;
};

codeChange = function(err, file, newCode) {
  if (err) {
    return notifier.error(err.toString(), {
      title: relativeName(err.file)
    });
  }
  if (!newCode) {
    return notifier.info("file unchanged", {
      title: relativeName(file) || srcMonitor.name
    });
  }
  notifier.info("file compiled sucessfully", {
    title: relativeName(file) || srcMonitor.name
  });
  if ((scriptName && !script) || (script && script.exitCode !== null)) {
    return start("Restarting " + scriptName);
  }
};

srcMonitor = new Monitor('src Monitor', path.resolve(project.config.src), project.filter);

srcMonitor.on('created', function(f) {
  if (project.fileFilter(f)) {
    return project.liveBuild(f, codeChange);
  }
});

srcMonitor.on('changed', function(f) {
  return project.liveBuild(f, codeChange);
});

srcMonitor.on('removed', function(f) {
  return project.removeBuild(f, codeChange);
});

srcMonitor.once('started', function(files) {
  notifier.info("Watching", {
    title: srcMonitor.name
  });
  return project.liveBuildAll(files, function(errors) {
    var e, _i, _len, _results;
    if (errors) {
      _results = [];
      for (_i = 0, _len = errors.length; _i < _len; _i++) {
        e = errors[_i];
        _results.push(notifier.error(e.toString(), {
          title: relativeName(e.file)
        }));
      }
      return _results;
    } else {
      notifier.info('Build done.', {
        title: srcMonitor.name
      });
      if (scriptName) {
        return start();
      }
    }
  });
});

srcMonitor.start();

/*
process stuff
*/


process.on('SIGINT', function(err) {
  if (script) {
    notifier.error('Killing script...', {
      title: scriptName
    });
    script.kill('SIGQUIT');
  }
  return process.exit();
});
